/*
 *	spCanvasP.h
 */

#ifndef __SPCANVASP_H
#define __SPCANVASP_H

#include <sp/spPrimitiveP.h>
#include <sp/spCanvas.h>
#if defined(GTK)
#include <sp/spCanvas_Gtk.h>
#elif defined(_WIN32) && !defined(USE_MOTIF)
#include <sp/spCanvas_Win.h>
#elif defined(ANDROID)
#include <sp/spCanvas_Android.h>
#elif defined(COCOA)
#include <sp/spCanvas_Cocoa.h>
#elif defined(MACOS)
#include <sp/spCanvas_Mac.h>
#else
#include <sp/spCanvas_Xm.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _spCanvasPart {
    spCanvasArchPart arch_part;
    
    int prev_width;
    int prev_height;
    spBool capture_flag;
    spBool exposed;
    spBool refreshed;

    spBool border_on;
    spBool focusable;
    spBool draw_background;
    spBool use_tab_key;
    spBool use_arrow_key;
    int size;

    spBool accept_wheel_event;
    spBool wheel_target_horizontal;
    spBool scroll_left_by_wheel_down;
    spMouseButton wheel_substitutive_button;
    
    spBool h_scroll;
    int h_value;
    int h_minimum;
    int h_maximum;
    int h_page_size;
    int h_increment;
    int h_page_increment;
    int h_wheel_increment;
    int h_scroll_coef;
    double h_content_factor;
    
    spBool v_scroll;
    int v_value;
    int v_minimum;
    int v_maximum;
    int v_page_size;
    int v_increment;
    int v_page_increment;
    int v_wheel_increment;
    int v_scroll_coef;
    double v_content_factor;
} spCanvasPart;

typedef struct _spCanvasRec {
    spObjectPart object;
    spComponentPart component;
    spPrimitivePart primitive;
    spCanvasPart canvas;
} spCanvasRec;
    
typedef struct _spCanvasClassPart {
    spBool (*refresh) __P((spComponent component));
    spBool (*redraw) __P((spComponent component));
} spCanvasClassPart;

typedef struct _spCanvasClassRec {
    spObjectClassPart object;
    spComponentClassPart component;
    spPrimitiveClassPart primitive;
    spCanvasClassPart canvas;
} spCanvasClassRec;

#if defined(MACOS)
#pragma import on
#endif

extern spCanvasClassRec SpCanvasClassRec;

#define SpCanvasPart(comp) (((spCanvas)comp)->canvas)
#define SpCanvasArch(comp) (((spCanvas)(comp))->canvas.arch_part)
#define SpParentCanvasPart(comp) (((spCanvas)SpGetParent(comp))->canvas)
#define SpGetCanvasClass(comp) ((spCanvasClass)((comp)->object.object_class))

extern void spCanvasPartInit(spObject object);
extern void spCanvasPartFree(spObject object);
extern spBool spCanvasCreate(spObject object);
extern void spCanvasCreateArch(spComponent component);
extern spBool spCanvasSetParams(spObject object);
extern void spCanvasSetParamsArch(spComponent component);
extern spBool spCanvasDestroy(spObject object);
extern void spCanvasDestroyArch(spComponent component);
extern spBool spIsCanvasExposed(spComponent component);
extern spBool spIsCanvasExposedArch(spComponent component);
extern void spExposeCanvasCB(spComponent component, void *data);
extern void spResizeCanvasCB(spComponent component, void *data);
extern void spExposeCanvasCBArch(spComponent component);
extern spBool spUpdateCanvasScrollBarArch(spComponent component);
extern void spCanvasSetDefaultSize(spComponent component);
extern spBool spCanvasRefresh(spComponent component);
extern spBool spRefreshCanvasArch(spComponent component);
extern spBool spCanvasRedraw(spComponent component);
extern spBool spRedrawCanvasArch(spComponent component);
extern spBool spSetCanvasCaptureArch(spComponent window);
extern spBool spSetCanvasCursorArch(spComponent component, spCursor cursor);
extern spBool spUnsetCanvasCursorArch(spComponent component);
extern spBool spIsCanvasFocusedArch(spComponent component);
extern spBool spIsCanvasCleared(spComponent component);
#if defined(SP_SUPPORT_PRINT_CANVAS)
extern spBool spPrintCanvasCreate(spComponent component);
extern spBool spPrintCanvasCreateArch(spComponent component);
#endif

extern double spZoomScrollCanvasPageSize(int *page_size, int *value, int minimum, int maximum, int client_size,
					 double content_factor, double relative_factor, spBool limit_flag);
extern spBool spUpdateScrollCanvasPageSize(spComponent component, double relative_factor, spBool limit_flag);
extern spBool spUpdateCanvasContentFactor(spComponent component, double relative_factor, spBool limit_flag);
extern spBool spZoomScrollCanvasLimit(spComponent component, double relative_factor, spBool limit_flag);
extern spBool spZoomScrollCanvasArch(spComponent component);
extern void spMoveCanvasScrollBarValue(int *value, double delta, int minimum, int maximum, int page_size, int wheel_increment);

extern spCallback *spFindDrawCallbackArch(spComponent component, long *index, spBool resize_flag);
extern spBool spEmitDrawCallback(spComponent component);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPCANVASP_H */
