/*
 *	spDIBitmapP.h
 */

#ifndef __SPDIBITMAPP_H
#define __SPDIBITMAPP_H

#include <sp/spDIBitmap.h>
#if defined(GTK)
#include <sp/spDIBitmap_Gtk.h>
#elif defined(_WIN32) && !defined(USE_MOTIF)
#include <sp/spDIBitmap_Win.h>
#elif defined(ANDROID)
#include <sp/spDIBitmap_Android.h>
#elif defined(COCOA)
#include <sp/spDIBitmap_Cocoa.h>
#elif defined(MACOS)
#include <sp/spDIBitmap_Mac.h>
#else
#include <sp/spDIBitmap_Xm.h>
#endif

#if defined(_WIN32)
#include <windows.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if !defined(_WIN32)
typedef unsigned long DWORD;  /* internal format (64bit unsigned long is OK) */
typedef unsigned short WORD;
typedef unsigned char BYTE;
typedef long LONG;            /* internal format (64bit long is OK) */

typedef struct tagBITMAPINFOHEADER {
    DWORD  biSize; 
    LONG   biWidth; 
    LONG   biHeight; 
    WORD   biPlanes; 
    WORD   biBitCount; 
    DWORD  biCompression; 
    DWORD  biSizeImage; 
    LONG   biXPelsPerMeter; 
    LONG   biYPelsPerMeter; 
    DWORD  biClrUsed; 
    DWORD  biClrImportant; 
} BITMAPINFOHEADER, *PBITMAPINFOHEADER;

typedef LONG FXPT16DOT16, *LPFXPT16DOT16;
typedef LONG FXPT2DOT30, *LPFXPT2DOT30;

typedef struct tagCIEXYZ {
    FXPT2DOT30 ciexyzX;
    FXPT2DOT30 ciexyzY;
    FXPT2DOT30 ciexyzZ;
} CIEXYZ;

typedef struct tagCIEXYZTRIPLE {
    CIEXYZ ciexyzRed; 
    CIEXYZ ciexyzGreen; 
    CIEXYZ ciexyzBlue; 
} CIEXYZTRIPLE;

typedef struct {
    DWORD        bV4Size;
    LONG         bV4Width;
    LONG         bV4Height;
    WORD         bV4Planes;
    WORD         bV4BitCount;
    DWORD        bV4V4Compression;
    DWORD        bV4SizeImage;
    LONG         bV4XPelsPerMeter;
    LONG         bV4YPelsPerMeter;
    DWORD        bV4ClrUsed;
    DWORD        bV4ClrImportant;
    DWORD        bV4RedMask;
    DWORD        bV4GreenMask;
    DWORD        bV4BlueMask;
    DWORD        bV4AlphaMask;
    DWORD        bV4CSType;
    CIEXYZTRIPLE bV4Endpoints;
    DWORD        bV4GammaRed;
    DWORD        bV4GammaGreen;
    DWORD        bV4GammaBlue;
} BITMAPV4HEADER, *PBITMAPV4HEADER;

typedef struct {
    DWORD        bV5Size;
    LONG         bV5Width;
    LONG         bV5Height;
    WORD         bV5Planes;
    WORD         bV5BitCount;
    DWORD        bV5Compression;
    DWORD        bV5SizeImage;
    LONG         bV5XPelsPerMeter;
    LONG         bV5YPelsPerMeter;
    DWORD        bV5ClrUsed;
    DWORD        bV5ClrImportant;
    DWORD        bV5RedMask;
    DWORD        bV5GreenMask;
    DWORD        bV5BlueMask;
    DWORD        bV5AlphaMask;
    DWORD        bV5CSType;
    CIEXYZTRIPLE bV5Endpoints;
    DWORD        bV5GammaRed;
    DWORD        bV5GammaGreen;
    DWORD        bV5GammaBlue;
    DWORD        bV5Intent;
    DWORD        bV5ProfileData;
    DWORD        bV5ProfileSize;
    DWORD        bV5Reserved;
} BITMAPV5HEADER, *PBITMAPV5HEADER;
#endif

struct _spDIBitmapFuncList
{
    int (*get_count)(void);
    const char *(*get_name)(int index);
    
    void *(*init_instance)(const char *name);
    spBool (*free_instance)(void *instance);
    
    int (*get_priority)(void *instance);
    const char **(*get_file_type_list)(void *instance);
    const char **(*get_file_desc_list)(void *instance);
    const char **(*get_file_filter_list)(void *instance);
    spBool (*is_supported)(void *instance, const char *filename);
    spBool (*set_file_type)(void *instance, int index);
    int (*get_file_type)(void *instance);
    unsigned long (*get_meta_info_mask)(void *instance);
    spBool (*set_meta_info)(void *instance, spMetaInfo *meta_info);
    spBool (*get_meta_info)(void *instance, spMetaInfo *meta_info);
    const char *(*get_best_suffix)(void *instance);
    
    unsigned long (*get_caps)(void *instance);
    
    spBool (*open)(void *instance, const char *filename, const char *mode);
    spBool (*close)(void *instance);
    
    spBool (*set_file_info)(void *instance, spDIBitmapInfo *info, unsigned long *strict_mask);
    spBool (*set_info)(void *instance, spDIBitmapInfoInternal *info);
    
    spBool (*get_file_info)(void *instance, spDIBitmapInfo *info);
    spBool (*get_info)(void *instance, spDIBitmapInfoInternal *info, unsigned long *strict_mask);
    
    void *(*read)(void *instance, spDIBitmapInfoInternal *info, void *bitmap);
    spBool (*write)(void *instance, spDIBitmapInfoInternal *info, void *bitmap);
    
    spBool (*read_rgba)(void *instance, int *r, int *g, int *b, int *a);
    spBool (*write_rgba)(void *instance, int r, int g, int b, int a);
};
    
struct _spDIBitmapHandler {
    spDIBitmapFuncList *func_list;
    char *name;
    unsigned long caps;
    int file_type_index;
    
    void *instance;
    spDIBitmapInfo file_info;
};
    
struct _spDIBitmapPluginRec
{
    void *host_data;
    void *user_data;
    
    spPluginType type;
    const char *plugin_id;
    long version_id;
    int priority;
    unsigned long caps;
    spBool (*init)(const char *lang);
    spBool (*free)(void);
    const char *description;
    const char *information;
    
    void *(*init_instance)(const char *lang);
    spBool (*free_instance)(void *instance);
    spOptions (*init_options)(void *instance, const char *lang);
    spBool (*free_options)(void *instance, spOptions options);
    
    spPluginDeviceType device_type;
    
    const char **file_type_list;
    const char **file_desc_list;
    const char **file_filter_list;
    spBool (*is_supported)(const char *filename);
    spBool (*set_file_type)(void *instance, int index);
    int (*get_file_type)(void *instance);
    unsigned long meta_info_mask;
    spBool (*set_meta_info)(void *instance, spMetaInfo *meta_info);
    spBool (*get_meta_info)(void *instance, spMetaInfo *meta_info);
    const char *(*get_best_suffix)(void *instance); 		/* suffix for file such as ".png", ".jpg" */

    unsigned long (*get_dib_caps)(void *instance);
    
    spPluginError (*open)(void *instance, const char *filename, const char *mode);
    spBool (*close)(void *instance);

    /* set_file_dib_info, set_dib_info, get_file_dib_info and get_dib_info may be called multiple times. */
    /* Unchangeable parameter masks of strict_mask of set_file_dib_info and get_dib_info must be updated in plug-in.
     * Don't update the masks if the parameters associated with the masks are changeable in the next function call. */
    spBool (*set_file_dib_info)(void *instance, spDIBitmapInfo *file_dibinfo, unsigned long *strict_mask);
    /* dibinfo is the same as the arg of read/write */
    spBool (*set_dib_info)(void *instance, spDIBitmapInfoInternal *dibinfo);
    
    spBool (*get_file_dib_info)(void *instance, spDIBitmapInfo *file_dibinfo); /* read only */
    spBool (*get_dib_info)(void *instance, spDIBitmapInfoInternal *dibinfo, unsigned long *strict_mask);
    
    spBool (*set_other_info)(void *instance, const char *id, char *data);
    spBool (*get_other_info)(void *instance, const char *id, char *data);
    
    spBool (*read)(void *instance, spDIBitmapInfoInternal *dibinfo, unsigned char *data);
    spBool (*write)(void *instance, spDIBitmapInfoInternal *dibinfo, unsigned char *data);

    spBool (*read_rgba)(void *instance, int *r, int *g, int *b, int *a); /* alternate use of read and read_rgba is not accepted */
    spBool (*write_rgba)(void *instance, int r, int g, int b, int a); /* alternate use of write and write_rgba is not accepted */
};
    
#if defined(MACOS)
#pragma import on
#if !defined(COCOA)
extern void *spCreateDIBitmapMac(spDIBitmapInfoInternal *info, GWorldPtr gworld);
#endif
#endif

extern spDIBitmapFuncList *spGetDefaultDIBitmapFuncList(void);
extern spDIBitmapFuncList *spGetPluginDIBitmapFuncList(void);
    
extern spDIBitmap spCreateDIBitmapCore(spDIBitmapInfo *info, spDIBitmapHandler handler, unsigned long options);
    
extern spBool spUpdateDIBitmapInfoArch(spDIBitmapInfoInternal *info, unsigned long *options);
extern void *spCreateDIBitmapArch(spDIBitmapInfoInternal *info, unsigned long options);
extern spBool spDestroyDIBitmapArch(void *bitmap);
    
#if defined(MACOS) || defined(ANDROID) || defined(GTK3)
extern spBool spLockDIBitmapArch(spDIBitmapInfoInternal *info, void *bitmap);
extern spBool spUnlockDIBitmapArch(spDIBitmapInfoInternal *info, void *bitmap);
#else
#define spLockDIBitmapArch(info, bitmap) (SP_TRUE)
#define spUnlockDIBitmapArch(info, bitmap) (SP_TRUE)
#endif
    
extern spBool spSetDIBDataRGBAArch(spDIBitmapInfoInternal *info, void *dibdata, int x, int y, int r, int g, int b, int a);
extern spBool spAddDIBDataRGBAArch(spDIBitmapInfoInternal *info, void *dibdata, int x, int y, int r, int g, int b, int a);
extern spBool spGetDIBDataRGBAArch(spDIBitmapInfoInternal *info, void *dibdata, int x, int y, int *r, int *g, int *b, int *a);

extern spBool spCopyDIBitmapToImageArch(spDIBitmapInfoInternal *info, void *bitmap, spComponent dest, int src_x, int src_y,
					int width, int height, int dest_x, int dest_y);
extern spBool spStretchDIBitmapToImageArch(spDIBitmapInfoInternal *info, void *bitmap, spComponent dest,
					   int src_x, int src_y, int src_width, int src_height,
					   int dest_x, int dest_y, int dest_width, int dest_height);
extern void *spCopyImageToDIBitmapArch(spComponent src, spDIBitmapInfoInternal *info, void *bitmap, int src_x, int src_y,
				       int width, int height, int dest_x, int dest_y);
    
extern void *spInitDIBitmapInstanceArch(const char *name);
extern spBool spFreeDIBitmapInstanceArch(void *instance);
extern int spGetDIBitmapPriorityArch(void *instance);
extern unsigned long spGetDIBitmapCapsArch(void *instance);
extern const char **spGetDIBitmapFileTypeListArch(void *instance);
extern const char **spGetDIBitmapFileDescListArch(void *instance);
extern const char **spGetDIBitmapFileFilterListArch(void *instance);

extern spBool spIsDIBitmapSupportedArch(void *instance, const char *filename);
extern spBool spSetDIBitmapFileTypeArch(void *instance, int index);
extern int spGetDIBitmapFileTypeArch(void *instance);
extern const char *spGetDIBitmapBestSuffixArch(void *instance);
extern spBool spOpenDIBitmapFileArch(void *instance, const char *filename, const char *mode); /* returns file instance */
extern spBool spCloseDIBitmapFileArch(void *instance);
extern spBool spGetDIBitmapInfoArch(void *instance, spDIBitmapInfoInternal *info, unsigned long *options);
extern void *spReadDIBitmapArch(void *instance, spDIBitmapInfoInternal *info, void *bitmap);
extern spBool spWriteDIBitmapArch(void *instance, spDIBitmapInfoInternal *info, void *bitmap);

extern long spReadBmpFileHeader(DWORD *pbfSize, DWORD *pbfOffBits, FILE *fp);
extern long spReadBmpV5Header(BITMAPV5HEADER *header, DWORD bfOffBits, FILE *fp);
extern DWORD spCalcFileSizeFromBmpV5Header(BITMAPV5HEADER *header, DWORD *pbfOffBits);
extern long spWriteBmpFileHeader(DWORD bfSize, DWORD bfOffBits, FILE *fp);
extern long spWriteBmpV5Header(BITMAPV5HEADER *header, DWORD bfOffBits, FILE *fp);
extern long spWriteBmpV5Data(BITMAPV5HEADER *header, spDIPixelFormat pixel_format, long line_stride,
                             spBool upside_down, unsigned char *data, FILE *fp);
extern spBool spGetFileDIBInfoFromBmpV5HeaderToRead(BITMAPV5HEADER *header, spDIBitmapInfo *dibinfo);
extern spBool spSetFileDIBInfoToBmpV5HeaderToWrite(BITMAPV5HEADER *header, spDIBitmapInfo *dibinfo, unsigned long *strict_mask);
extern spBool spGetDIBInfoFromBmpV5Header(BITMAPV5HEADER *header, spDIBitmapInfoInternal *dibinfo, unsigned long *strict_mask, spBool write_flag);
extern long spReadBmpV5Data(BITMAPV5HEADER *header, spDIPixelFormat pixel_format, long line_stride,
                            spBool upside_down, unsigned char *data, FILE *fp);

extern spBool spReverseDIBitmapDefault(spDIBitmapInfoInternal *info, void *dibdata);
extern spBool spReverseDIBitmapArch(spDIBitmapInfoInternal *info, void *dibdata);
extern spBool spFlipDIBitmapDefault(spDIBitmapInfoInternal *info, void *dibdata, spBool lr_flag);
extern spBool spFlipDIBitmapArch(spDIBitmapInfoInternal *info, void *dibdata, spBool lr_flag);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPDIBITMAPP_H */
