/*
 *	spDrawingPluginP.h
 */

#ifndef __SPDRAWINGPLUGINP_H
#define __SPDRAWINGPLUGINP_H

#include <sp/spPluginP.h>
#include <sp/spDrawingPlugin.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    SP_POLYGON_EVENT_START_POINT = 0,	/* x, y: coordinate */
    SP_POLYGON_EVENT_ADD_POINT = 1,	/* x, y: coordinate */
    SP_POLYGON_EVENT_END_POINT = 2,	/* x, y: always 0 */
    SP_POLYGON_EVENT_MOVE = 3,		/* x, y: amount of moving */
} spPolygonEventType;

/* In callback case, functions called between start and end do not work.
 * All drawing will be done in the callback function. */
#define SP_DRAWING_PLUGIN_HOST_CR_DRAW SP_PLUGIN_HOST_CR_USER
#define SP_DRAWING_PLUGIN_HOST_CR_DRAW_PAGE (SP_PLUGIN_HOST_CR_USER+1) /* callback_data: int page_number (1-based) */

struct _spDrawingPluginRec
{
    void *host_data;
    void *user_data;
    
    spPluginType type;
    char *plugin_id;
    long version_id;
    int priority;
    unsigned long caps;
    spBool (*init)(const char *lang);
    spBool (*free)(void);
    char *description;
    char *information;
    
    void *(*init_instance)(const char *lang);
    spBool (*free_instance)(void *instance);
    spOptions (*init_options)(void *instance, const char *lang);
    spBool (*free_options)(void *instance, spOptions options);
    
    spDrawingDeviceType device_type;

    spBool (*get_num_device)(void *instance, int *num_device);
    const char *(*get_device_name)(void *instance, int index); 		/* short explanation */
    const char *(*get_device_description)(void *instance, int index); 	/* description */
    spBool (*select_device)(void *instance, int index);
    
    void (*set_parent)(void *instance, spComponent parent);
    spBool (*create_component)(void *instance, void *parent_arch, spArg *args, int num_arg);
    spBool (*destroy_component)(void *instance);
    spBool (*set_params)(void *instance, spArg *args, int num_arg);
    spBool (*get_params)(void *instance, spArg *args, int num_arg);

    spBool (*open_setup_dialog)(void *instance, spArg *args, int num_arg);

    /* set_page_info is prepared for setup using other plugin's info. */
    spBool (*set_page_info)(void *instance, spPaperSize paper_size, spPaperOrientation paper_orientation,
        double left_margin_mm, double right_margin_mm, double top_margin_mm, double bottom_margin_mm);
    spBool (*get_page_info)(void *instance, spPaperSize *paper_size, spPaperOrientation *paper_orientation,
        double *left_margin_mm, double *right_margin_mm, double *top_margin_mm, double *bottom_margin_mm);
    spBool (*set_page_range)(void *instance, int start_page, int end_page, int max_page); /* 1-based number */
    spBool (*get_page_range)(void *instance, int *start_page, int *end_page); /* 1-based number */
    spBool (*get_page_copy_info)(void *instance, int *num_copy, spBool *collate);

    /* for file (open_target_dialog can be NULL, if the following parameters are set) */
    char **file_type_list;
    char **file_desc_list;
    char **file_filter_list;
    spBool (*set_file_type)(void *instance, int index);
    const char *(*get_best_suffix)(void *instance); 		/* suffix for file such as ".eps", ".jpg" */

    /* one of the following functions must be implemented */
    spBool (*open_target_file)(void *instance, const char *filename);	/* case using file */
    spBool (*open_target_dialog)(void *instance, spArg *args, int num_arg); /* case displaying dialog */

    spBool (*close_target)(void *instance);

    spBool (*is_drawable)(void *instance);

    /* the following set_* functions must be called before start_drawing */
    unsigned long (*get_drawing_caps)(void *instance);
    unsigned long (*get_graphics_mode_caps)(void *instance);
    spBool (*set_size)(void *instance, int width, int height);
    spBool (*get_client_size)(void *instance, int *width, int *height);
    /* get_phisical_size obtains a phisical size in milli meter except margins */
    spBool (*get_physical_size)(void *instance, double *width_mm, double *height_mm);
    spBool (*set_scaling_factor)(void *instance, double xfactor, double yfactor);

    spBool (*start_drawing)(void *instance, const char *job_name);
    spBool (*end_drawing)(void *instance);
    spBool (*forward_page)(void *instance);

    /* src_image_arch: system-depend offline image such as memdc (Win) or pixmap (Xm, Gtk, Mac) */
    spBool (*refresh)(void *instance);
    spBool (*copy_image)(void *instance, void *src_image_arch,
			 int src_x, int src_y, int src_width, int src_height,
			 int dest_x, int dest_y, int dest_width, int dest_height);
    
    void (*fill_rectangle)(void *instance, spGraphics graphics,
			   int x, int y, int width, int height);
    void (*draw_rectangle)(void *instance, spGraphics graphics,
			   int x, int y, int width, int height);
    void (*fill_arc)(void *instance, spGraphics graphics,
		     int x, int y, int width, int height, double angle1, double angle2);
    void (*draw_arc)(void *instance, spGraphics graphics,
		     int x, int y, int width, int height, double angle1, double angle2);
    void (*draw_line)(void *instance, spGraphics graphics,
		      int x1, int y1, int x2, int y2);
    void (*draw_point)(void *instance, spGraphics graphics,
		       int x, int y);
    void (*draw_string)(void *instance, spGraphics graphics,
			int x, int y, const char *string);
    spBool (*get_string_extent)(void *instance, spGraphics graphics, const char *string,
				int *x, int *y, int *width, int *height, int *next_x);

    void *polygon_host_data;	/* must be NULL */
    void *(*init_polygon_instance)(spPolygon polygon);
    spBool (*free_polygon_instance)(void *polygon_instance);
    
    /* if init_polygon_instance is set to NULL, polygon_instance of following functions is spPolygon. */
    spBool (*polygon_event)(void *instance, void *polygon_instance, spPolygonEventType event_type, int x, int y);
    spBool (*fill_polygon)(void *instance, spGraphics graphics, void *polygon_instance);
    spBool (*draw_polygon)(void *instance, spGraphics graphics, void *polygon_instance);
};

#if defined(MACOS)
#pragma import on
#endif


#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPDRAWINGPLUGINP_H */
