/*
 *        spPrimitive_Gtk.h
 */

#ifndef __SPPRIMITIVE_GTK_H
#define __SPPRIMITIVE_GTK_H

#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif

#if GTK_CHECK_VERSION(3,0,0)
#define SP_USE_GTK_GRID
#define SP_SLIDER_VALUE_CHANGED_CALLBACK_NAME "change-value"
#else
#define SP_SLIDER_VALUE_CHANGED_CALLBACK_NAME "value_changed"
#endif
    
struct _spCallback {
    spComponent component;
    spCallbackFunc call_func;
    void *call_data;
    spBool propagate;
    
    const gchar *call_name;
    gint func_id;
    gint event_mask;
};

typedef struct _spCallbackTable {
    spCallbackType call_type;
    spComponentType component_type;
    const gchar *call_name;
    gint event_mask;
} spCallbackTable;

typedef struct _spPrimitiveArchPart {
    GtkWidget *widget;          /* main widget */
    GtkWidget *top_widget;      /* top widget in component */
    GtkWidget *sub_widget;      /* sub widget */
#if GTK_CHECK_VERSION(3,0,0)
    GtkAdjustment *adjustment;  /* adjustment widget */
    cairo_surface_t *pixmap;    /* pixmap for drawable */
    cairo_t *pixmap_cr;         /* context of pixmap */
#else
    GtkObject *adjustment;      /* adjustment widget */
    GdkPixmap *pixmap;          /* pixmap for drawable */
#endif
    int state;                  /* state for some widgets */
    int prev_value;
    const gchar *call_name;
    GdkEvent *event;
    gint event_mask;
    
#if GTK_CHECK_VERSION(2,0,0)
    PangoLayout *layout;
#endif

    int set_width;
    int set_height;
} spPrimitiveArchPart;
    
extern gboolean spIsWidgetMappedGtk(GtkWidget *widget);
extern gboolean spIsWidgetRealizedGtk(GtkWidget *widget);
extern gboolean spWidgetHasGrabGtk(GtkWidget *widget);
extern GdkWindow *spGetGdkWindowGtk(GtkWidget *widget);
    
extern gint spSignalConnectCallbackGtk(GtkWidget *widget, const gchar *call_name, spCallbackFuncGtk func, gpointer data);
extern gint spSignalConnectEventGtk(GtkWidget *widget, const gchar *call_name, spEventFuncGtk func, gpointer data);
extern void spEmitComponentCallbackFuncGtk(spCallback *callback, GdkEvent *event);
extern void spSetWidgetSizeGtk(GtkWidget *widget, int width, int height);
extern void spSetStyleGtk(spComponent component, GtkWidget *widget);
extern void spPrimitiveCreateBorderGtk(spComponent component);
extern void spContainerAddGtk(spComponent component, GtkWidget *widget);
extern void spPrimitiveContainerAddGtk(spComponent component);
#if GTK_CHECK_VERSION(2,0,0)
extern gboolean spEventsCallbackGtk(gpointer data);
#endif
    
#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPPRIMITIVE_GTK_H */
